mod table_directory;
mod character_map;
mod maximum_profile;
mod font_header;
mod index_to_location;
mod search;

use std::{fs::File, io::{BufReader, Read}};

use bincode::Options;
use character_map::CharacterMap;
use font_header::FontHeader;
use index_to_location::{IndexToLocation, IndexToLocationArgs};
use log::{debug, info};
use maximum_profile::MaximumProfile;
use table_directory::TableDirectory;

fn deserialize<R: Read, T: serde::de::DeserializeOwned>(reader: R) -> bincode::Result<T> {
	bincode::options().with_big_endian().with_fixint_encoding().deserialize_from::<R, T>(reader)
}

#[derive(Debug)]
pub struct Font {
	table_directory: TableDirectory,
	font_header: FontHeader,
	character_map: CharacterMap,
	maximum_profile: MaximumProfile,
	index_to_location: IndexToLocation
}
	
impl Font {
	pub fn new(file: File) -> bincode::Result<Self> {
		let mut buf_reader = BufReader::new(file.try_clone()?);
		let table_directory = TableDirectory::new(&mut buf_reader)?;

		let font_header: FontHeader = table_directory.get_table(&file)?;
		debug!("{font_header:#?}");

		let character_map = table_directory.get_table(&file)?;
		debug!("{character_map:#?}");

		let maximum_profile: MaximumProfile = table_directory.get_table(&file)?;
		debug!("{maximum_profile:#?}");

		let index_to_location = table_directory.get_table_with_args(&file, Some(IndexToLocationArgs {
			format: font_header.index_to_loc_format,
			num_glyphs: maximum_profile.num_glyphs()
		}))?;

		Ok(Font {
			table_directory,
			font_header,
			character_map,
			maximum_profile,
			index_to_location
		})
	}

	pub fn get_data_for(&mut self, glyph: char) -> Result<(), &'static str> {
		let Some(glyph_id) = self.character_map.get_char(glyph) else {
			return Err("Failed to get glyph id for char");
		};

		info!("glyph id for char '{glyph}' is {glyph_id}");

		let Some(offset) = self.index_to_location.get_offset(glyph_id) else {
			return Err("Failed to get glyph offset for char");
		};

		info!("glyph has offset {offset}");

		Ok(())
	}
}

